"""
parser.local package (imdb package).

This package provides the IMDbLocalAccessSystem class used to access
IMDb's data through a local installation.
the imdb.IMDb function will return an instance of this class when
called with the 'accessSystem' argument set to "local" or "files".

Copyright 2004 Davide Alberani <da@erlug.linux.it> 

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import os

from imdb import IMDbBase
from imdb._exceptions import IMDbDataAccessError, IMDbNotAvailable
from imdb.utils import analyze_title, analyze_name
from utils import getFullIndex, sortBy
from personParser import getFilmography, getBio
from movieParser import getLabel, getMovieCast, getAkaTitles, parseMinusList, \
                        getPlot, getRatingData, getMovieMisc, getTaglines
from ratober import search_name, search_title


class IMDbLocalAccessSystem(IMDbBase):
    """The class used to access IMDb's data through a local installation."""

    accessSystem = 'local'

    def __init__(self, dbDirectory):
        """Initialize the access system.
        The directory with the files must be supplied.
        """
        dbDirectory = os.path.expandvars(dbDirectory)
        dbDirectory = os.path.expanduser(dbDirectory)
        dbDirectory = os.path.realpath(dbDirectory)
        if not os.path.isdir(dbDirectory):
            raise IMDbDataAccessError, '"%s" is not a directory' % dbDirectory
        self.__db = dbDirectory + os.path.sep

    def _normalize_id(self, mopID):
        """Normalize the given movieID or personID."""
        return int(mopID)

    def _fetchIMDbPage(self, page):
        """Fetch the given web page from the IMDb akas server."""
        from imdb.parser.http import IMDbURLopener
        url = 'http://akas.imdb.com/%s' % page
        content = ''
        try:
            urlOpener = IMDbURLopener()
            uopener = urlOpener.open(url)
            content = uopener.read()
            uopener.close()
            urlOpener.close()
        except IOError:
            pass
        return content

    def get_imdbMovieID(self, movieID):
        """Translate a movieID in an imdbID.
        Try accessing the /Title?Title+(year) page on IMDb;
        return None if it's unable to get the imdbID.
        """
        # FIXME: it will probably NOT work most of the times.
        #        Do a search for the long imdb title and check
        #        if there's only one "exact match"?
        titline = getLabel(movieID, '%stitles.index' % self.__db,
                            '%stitles.key' % self.__db)
        if not titline: return None
        titledic = analyze_title(titline)
        if not titledic.has_key('title'): return None
        title = titledic['title']
        year = titledic.get('year')
        if year:
            title += ' (%s' % year
            if titledic.has_key('imdbIndex'):
                title += '/%s' % titledic['imdbIndex']
            title += ')'
        from urllib import quote_plus
        page = 'Title?%s' % quote_plus(title)
        content = self._fetchIMDbPage(page)
        if not content: return None
        from imdb.parser.http.searchMovieParser import BasicMovieParser
        mparser = BasicMovieParser()
        result = mparser.parse(content)
        if not result: return None
        return result[0][0]

    def get_imdbPersonID(self, personID):
        """Translate a personID in an imdbID.
        Try accessing the /Name?Surname,+Name page on IMDb;
        return None if it's unable to get the imdbID.
        """
        name = getLabel(personID, '%snames.index' % self.__db,
                        '%snames.key' % self.__db)
        if not name: return None
        from urllib import quote_plus
        page = 'Name?%s' % quote_plus(name)
        content = self._fetchIMDbPage(page)
        if not content: return None
        from imdb.parser.http.searchPersonParser import BasicPersonParser
        pparser = BasicPersonParser()
        result = pparser.parse(content)
        if not result: return None
        return result[0][0]

    def _search_movie(self, title, results):
        # ratober functions return a sorted
        # list of tuples (match_score, movieID, movieTitle)
        return [(x[1], analyze_title(x[2]))
                for x in search_title(title.strip(),
                '%stitles.key' % self.__db, results)]

    def get_movie_main(self, movieID):
        # Information sets provided by this method.
        infosets = ('main', 'vote details')
        res = analyze_title(getLabel(movieID,
                            '%stitles.index' % self.__db,
                            '%stitles.key' % self.__db))
        # No title, no party.
        if not res: return ({}, infosets)
        # Build the cast list.
        actl = []
        for castG in ('actors', 'actresses'):
            midx = getFullIndex('%s%s.titles' % (self.__db, castG),
                            movieID, multi=1)
            if midx:
                params = {'movieID': movieID,
                            'dataF': '%s%s.data' % (self.__db, castG),
                            'indexF': '%snames.index' % self.__db,
                            'keyF': '%snames.key' % self.__db,
                            'attrIF': '%sattributes.index' % self.__db,
                            'attrKF': '%sattributes.key' % self.__db,
                            'offsList': midx, 'doCast': 1}
                actl += getMovieCast(**params)
        if actl:
            actl[:] = sortBy(actl, 'position')
            res['cast'] = actl
        # List of other workers.
        works = ('producer', 'writer', 'cinematographer', 'composer',
                'costume-designer', 'director', 'editor', 'miscellaneou',
                'producer', 'production-designer', 'cinematographer')
        for i in works:
            index = getFullIndex('%s%ss.titles' % (self.__db, i),
                                    movieID, multi=1)
            if index:
                params = {'movieID': movieID,
                            'dataF': '%s%s.data' % (self.__db, i),
                            'indexF': '%snames.index' % self.__db,
                            'keyF': '%snames.key' % self.__db,
                            'attrIF': '%sattributes.index' % self.__db,
                            'attrKF': '%sattributes.key' % self.__db,
                            'offsList': index}
                name = key = i
                if '-' in name:
                    name = name.replace('-', ' ')
                elif name == 'miscellaneou':
                    name = 'crewmembers'
                    key = 'miscellaneou'
                elif name == 'writer':
                    params['doWriters'] = 1
                params['dataF'] = '%s%ss.data' % (self.__db, key)
                data = getMovieCast(**params)
                if name == 'writer': data[:] = sortBy(data, 'order')
                res[name] = data
        # Rating.
        rt = self.get_movie_vote_details(movieID)
        if rt: res.update(rt)
        # Various information.
        miscInfo = (('runtimes', 'running-times'), ('color info', 'color-info'),
                    ('genres', 'genres'), ('distributors', 'distributors'),
                    ('languages', 'language'), ('certificates', 'certificates'),
                    ('special effects companies', 'special-effects-companies'),
                    ('sound mix', 'sound-mix'), ('tech info', 'technical'),
                    ('production companies', 'production-companies'),
                    ('countries', 'countries'))
        for name, fname in miscInfo:
            params = {'movieID': movieID,
                'dataF': '%s%s.data' % (self.__db, fname),
                'indexF': '%s%s.index' % (self.__db, fname),
                'attrIF': '%sattributes.index' % self.__db,
                'attrKF': '%sattributes.key' % self.__db}
            data = getMovieMisc(**params)
            if data: res[name] = data
        # AKA titles.
        akas = getAkaTitles(movieID,
                    '%saka-titles.data' % self.__db,
                    '%stitles.index' % self.__db,
                    '%stitles.key' % self.__db,
                    '%sattributes.index' % self.__db,
                    '%sattributes.key' % self.__db)
        if akas: res['akas'] = akas
        return (res, infosets)

    def get_movie_plot(self, movieID):
        pl = getPlot(movieID, '%splot.index' % self.__db,
                                '%splot.data' % self.__db)
        if pl: return {'plot': pl}
        return {}

    def get_movie_awards(self, movieID):
        raise IMDbNotAvailable, 'awards information not available ' + \
                                'in the local database'

    def get_movie_taglines(self, movieID):
        tg = getTaglines(movieID, '%staglines.index' % self.__db,
                        '%staglines.data' % self.__db)
        if tg: return {'taglines': tg}
        return {}

    def get_movie_keywords(self, movieID):
        params = {'movieID': movieID,
            'dataF': '%skeywords.data' % self.__db,
            'indexF': '%skeywords.index' % self.__db,
            'attrIF': '%sattributes.index' % self.__db,
            'attrKF': '%sattributes.key' % self.__db}
        kwds = getMovieMisc(**params)
        if kwds: return {'keywords': kwds}
        return {}

    def get_movie_alternate_versions(self, movieID):
        av = parseMinusList(movieID, '%salternate-versions.data' % self.__db,
                        '%salternate-versions.index' % self.__db)
        if av: return {'alternate versions': av}
        return {}

    def get_movie_crazy_credits(self, movieID):
        cc = parseMinusList(movieID, '%scrazy-credits.data' % self.__db,
                            '%scrazy-credits.index' % self.__db)
        if cc: return {'crazy credits': cc}
        return {}

    def get_movie_goofs(self, movieID):
        goo = parseMinusList(movieID, '%sgoofs.data' % self.__db,
                            '%sgoofs.index' % self.__db)
        if goo: return {'goofs': goo}
        return {}

    def get_movie_quotes(self, movieID):
        mq = parseMinusList(movieID, '%squotes.data' % self.__db,
                            '%squotes.index' % self.__db)
        if mq: return {'quotes': mq}
        return {}

    def get_movie_release_dates(self, movieID):
        params = {'movieID': movieID,
            'dataF': '%srelease-dates.data' % self.__db,
            'indexF': '%srelease-dates.index' % self.__db,
            'attrIF': '%sattributes.index' % self.__db,
            'attrKF': '%sattributes.key' % self.__db}
        data = getMovieMisc(**params)
        if data: return {'release dates': data}
        return {}

    def get_movie_vote_details(self, movieID):
        return getRatingData(movieID, '%sratings.data' % self.__db)

    def get_movie_official_sites(self, movieID):
        raise IMDbNotAvailable, 'official sites information not available ' + \
                                'in the local database'

    def get_movie_trivia(self, movieID):
        triv = parseMinusList(movieID, '%strivia.data' % self.__db,
                            '%strivia.index' % self.__db)
        if triv: return {'trivia': triv}
        return {}

    def get_movie_locations(self, movieID):
        params = {'movieID': movieID,
            'dataF': '%slocations.data' % self.__db,
            'indexF': '%slocations.index' % self.__db,
            'attrIF': '%sattributes.index' % self.__db,
            'attrKF': '%sattributes.key' % self.__db}
        data = getMovieMisc(**params)
        if data: return {'locations': data}
        return {}

    def get_movie_connections(self, movieID):
        # XXX: not sure.  check the movie-links.data file.
        raise IMDbNotAvailable, 'movie connections information not ' + \
                                'available in the local database'
    
    def _search_person(self, name, results):
        # ratober functions return a sorted
        # list of tuples (match_score, personID, personName)
        return [(x[1], analyze_name(x[2]))
                for x in search_name(name.strip(),
                '%snames.key' % self.__db, results)]

    def get_person_main(self, personID):
        infosets = ('main', 'biography', 'other works')
        res = analyze_name(getLabel(personID,
                            '%snames.index' % self.__db,
                            '%snames.key' % self.__db))
        # No name, no party.
        if not res: return ({}, infosets)
        res.update(getBio(personID, '%sbiographies.index' % self.__db,
                    '%sbiographies.data' % self.__db))
        return (res, infosets)

    def get_person_filmography(self, personID):
        res = {}
        works = ('actor', 'actresse', 'producer', 'writer',
                'cinematographer', 'composer', 'costume-designer',
                'director', 'editor', 'miscellaneou', 'producer',
                'production-designer')
        for i in works:
            index = getFullIndex('%s%ss.names' % (self.__db, i), personID)
            if index is not None:
                params = {'offset': index,
                            'indexF': '%stitles.index' % self.__db,
                            'keyF': '%stitles.key' % self.__db,
                            'attrIF': '%sattributes.index' % self.__db,
                            'attrKF': '%sattributes.key' % self.__db}
                name = key = i
                if '-' in name:
                    name = name.replace('-', ' ')
                elif name == 'actresse':
                    name = 'actress'
                    params['doCast'] = 1
                elif name == 'miscellaneou':
                    name = 'crewmembers'
                    key = 'miscellaneou'
                elif name == 'actor':
                    params['doCast'] = 1
                elif name == 'writer':
                    params['doWriters'] = 1
                params['dataF'] = '%s%ss.data' % (self.__db, key)
                data = getFilmography(**params)
                data[:] = sortBy(data, 'year', remove=0, reverse=1)
                res[name] = data
        return res

    def get_person_biography(self, personID):
        return self.get_person_main(personID)

    def get_person_awards(self, personID):
        raise IMDbNotAvailable, 'awards information not available ' + \
                                'in the local database'

    def get_person_other_works(self, personID):
        return self.get_person_main(personID)

    def get_person_official_sites(self, personID):
        raise IMDbNotAvailable, 'official sites information not available ' + \
                                'in the local database'


